@@ -1,6 +1,6 @@
 # sourceCpp("inseq.cpp")
 
-mcse.initseq <- function(x, adjust = FALSE, g = NULL, level = 0.95)
+mcse.initseq <- function(x, g = NULL, adjust = FALSE, blather = FALSE)
 { 
   
   chain <- as.matrix(x)
@@ -42,27 +42,18 @@ mcse.initseq <- function(x, adjust = FALSE, g = NULL, level = 0.95)
   {
     sig.adj <- NULL
   }
+
   
-  #calculate volume of the confidence region to the pth root
-  crit <- qchisq(level,df=p)/n
-  #log scale
-  foo <- -(log(p/2) + lgamma(p/2))/p + log(pi*crit)/2
-  det2p <- log(det(sig))/2/p
-  ##vol=volume to the pth root without adjustment
-  vol <- exp(foo + det2p)
-  
-  ##vol.adj=volume to the pth root with adjustment, if adjust=T
-  ##       =NULL, if adjust=F
-  if(adjust)
+  if(blather)
   {
-    det2p.adj <- log(det(sig.adj))/2/p
-    vol.adj <- exp(foo + det2p.adj)
-  }else
-  {
-    vol.adj <- NULL
+    return(list("cov" = sig, "cov.adj"=sig.adj,
+              "est" = mu.hat, "nsim" = n, "adjust" = adjust)) 
+  } else{
+    return(list("cov" = sig, "cov.adj"=sig.adj,
+              "est" = mu.hat)) 
   }
-  
-  return(list("cov" = sig, "cov.adj"=sig.adj,
-              "vol"=vol, "vol.adj"=vol.adj,
-              "est" = mu.hat, "nsim" = n, "adjust" = adjust))
-}
\ No newline at end of file
+
+}
+
+
+
